package eu.etaxonomy.taxeditor.navigation.navigator;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.ui.internal.navigator.dnd.CommonDropAdapterDescriptor;
import org.eclipse.ui.internal.navigator.dnd.CommonDropDescriptorManager;
import org.eclipse.ui.internal.navigator.extensions.CommonDragAssistantDescriptor;
import org.eclipse.ui.internal.navigator.extensions.NavigatorViewerDescriptor;
import org.eclipse.ui.navigator.CommonDragAdapterAssistant;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;
import org.eclipse.ui.navigator.INavigatorContentService;
import org.eclipse.ui.navigator.INavigatorDnDService;

import eu.etaxonomy.taxeditor.navigation.navigator.dnd.TaxonNavigatorDragAdapterAssistant;

public class TaxonNavigatorDnDService implements INavigatorDnDService {
	private static final CommonDropAdapterAssistant[] NO_ASSISTANTS = new CommonDropAdapterAssistant[0];

	private INavigatorContentService contentService;

	private TaxonNavigatorDragAdapterAssistant[] dragAssistants;
	
	private TreeNodeDropAdapter dropAdapter;

	private final Map dropAssistants = new HashMap();
	
	public TaxonNavigatorDnDService(INavigatorContentService contentService){
		this.contentService = contentService;
	}
	
	
	@Override
	public CommonDragAdapterAssistant[] getCommonDragAssistants() {
		
		return dragAssistants;
	}

	@Override
	public void bindDragAssistant(CommonDragAdapterAssistant anAssistant) {
		if(dragAssistants == null) 
			initializeDragAssistants(); 
		TaxonNavigatorDragAdapterAssistant[] newDragAssistants = new TaxonNavigatorDragAdapterAssistant[dragAssistants.length + 1];
		System.arraycopy(dragAssistants, 0, newDragAssistants, 0, dragAssistants.length);
		newDragAssistants[dragAssistants.length] = (TaxonNavigatorDragAdapterAssistant) anAssistant;
		dragAssistants = newDragAssistants;		

	}
	
	private void initializeDragAssistants() {
		int i = 0;
		Set dragDescriptors = ((NavigatorViewerDescriptor) contentService
				.getViewerDescriptor()).getDragAssistants();
		dragAssistants = new TaxonNavigatorDragAdapterAssistant[dragDescriptors
				.size()];
		for (Iterator iter = dragDescriptors.iterator(); iter.hasNext();) {
			CommonDragAssistantDescriptor descriptor = (CommonDragAssistantDescriptor) iter
					.next();
			dragAssistants[i++] = (TaxonNavigatorDragAdapterAssistant) descriptor.createDragAssistant();
		}
	}

	@Override
	public CommonDropAdapterAssistant[] findCommonDropAdapterAssistants(
			Object aDropTarget, TransferData aTransferType) {
		CommonDropAdapterDescriptor[] descriptors = CommonDropDescriptorManager
				.getInstance().findCommonDropAdapterAssistants(aDropTarget,
						contentService);

		if (descriptors.length == 0) {
			return NO_ASSISTANTS;
		}

		if (LocalSelectionTransfer.getTransfer().isSupportedType(aTransferType)  
						&& LocalSelectionTransfer.getTransfer().getSelection() instanceof IStructuredSelection) {
			return getAssistantsBySelection(descriptors, (IStructuredSelection) LocalSelectionTransfer.getTransfer().getSelection());
		} 
		return getAssistantsByTransferData(descriptors, aTransferType);
	}
	
	private CommonDropAdapterAssistant[] getAssistantsByTransferData(
			CommonDropAdapterDescriptor[] descriptors,
			TransferData aTransferType) {

		Set assistants = new LinkedHashSet();
		for (int i = 0; i < descriptors.length; i++) {
			CommonDropAdapterAssistant asst = getAssistant(descriptors[i]);
			if (asst.isSupportedType(aTransferType)) {
				assistants.add(asst);
			}
		}
		return sortAssistants((CommonDropAdapterAssistant[]) assistants
				.toArray(new CommonDropAdapterAssistant[assistants.size()]));

	}
	
	private CommonDropAdapterAssistant[] getAssistantsBySelection(
			CommonDropAdapterDescriptor[] descriptors, IStructuredSelection aSelection) {

		Set assistants = new LinkedHashSet(); 
			
		for (int i = 0; i < descriptors.length; i++) {
			if(descriptors[i].areDragElementsSupported(aSelection)) {
				assistants.add(getAssistant(descriptors[i]));
			}
		}  

		return sortAssistants((CommonDropAdapterAssistant[]) assistants
				.toArray(new CommonDropAdapterAssistant[assistants.size()]));
	}
	
	private CommonDropAdapterAssistant getAssistant(
			CommonDropAdapterDescriptor descriptor) {
		TreeNodeDropAdapterAssistant asst = (TreeNodeDropAdapterAssistant) dropAssistants
				.get(descriptor);
		if (asst != null) {
			return asst;
		}
		synchronized (dropAssistants) {
			asst = (TreeNodeDropAdapterAssistant) dropAssistants.get(descriptor);
			if (asst == null) {
				dropAssistants.put(descriptor, (asst = (TreeNodeDropAdapterAssistant) descriptor
						.createDropAssistant()));
				asst.init(contentService);
				//asst.setCommonDropAdapter(dropAdapter);
			}
		}
		return asst;
	}


	private CommonDropAdapterAssistant[] sortAssistants(CommonDropAdapterAssistant[] array) {
		Arrays.sort(array, new Comparator() {
			public int compare(Object arg0, Object arg1) {
				CommonDropAdapterAssistant a = (CommonDropAdapterAssistant) arg0;
				CommonDropAdapterAssistant b = (CommonDropAdapterAssistant) arg1;
				// This is to ensure that the navigator resources drop assistant will
				// always be first on the list of drop assistant, if a conflict ever 
				// occurs.
				String id = "org.eclipse.ui.navigator.resources."; //$NON-NLS-1$
				if (a.getClass().getName().startsWith(id))
					return -1;
				if (b.getClass().getName().startsWith(id))
					return 1;
				return a.getClass().getName().compareTo(b.getClass().getName());
			}
		});
		return array;
	}

	@Override
	public CommonDropAdapterAssistant[] findCommonDropAdapterAssistants(
			Object aDropTarget, IStructuredSelection theDragSelection) {
		// TODO Auto-generated method stub
		return null;
	}

	public void setDropAdaptor(TreeNodeDropAdapter da) {
		dropAdapter = da;
	}
}
