/**
* Copyright (C) 2015 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.navigation.navigator.handler;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.api.conversation.ConversationHolderMock;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.model.AbstractUtility;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigatorLabels;
import eu.etaxonomy.taxeditor.navigation.navigator.operation.RemotingMoveFactualDataOperation;
import eu.etaxonomy.taxeditor.operation.RemotingCdmHandler;
import eu.etaxonomy.taxeditor.ui.dialog.selection.TaxonNodeSelectionDialog;

/**
 * @author cmathew
 * @date 19 Jun 2015
 *
 */
public class RemotingMoveFactualDataHandler extends RemotingCdmHandler {


    private TaxonNode sourceTaxonNode;
    private TaxonNode targetTaxonNode;
    /**
     * @param label
     */
    public RemotingMoveFactualDataHandler() {
        super(TaxonNavigatorLabels.MOVE_FACTUAL_DATA_LABEL);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#allowOperations(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public IStatus allowOperations(ExecutionEvent event) {
        TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
        // check that only a single taxon tree node has been selected
        if(selection.size() > 1) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SINGLE_TAXON_SELECTION_MESSAGE);
        }

        // check for no taxon tree node selected
        if(selection.size() == 0) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.NO_TAXON_SELECTION_MESSAGE);
        }

        // check that selected object is a taxon node
        Object obj = selection.iterator().next();
        if(obj instanceof ITaxonTreeNode) {
            sourceTaxonNode = (TaxonNode)obj;
        } else {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.SELECTED_OBJECT_NOT_TREE_NODE_MESSAGE);
        }

        if(NavigationUtil.isDirty(sourceTaxonNode)) {
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.UNSAVED_CHANGES_MESSAGE);
        }

        List<UUID> excludeTaxa = new ArrayList<UUID>();
        excludeTaxa.add(sourceTaxonNode.getTaxon().getUuid());

        targetTaxonNode = TaxonNodeSelectionDialog.select(HandlerUtil.getActiveShell(event),
                new ConversationHolderMock(),
                "Choose the accepted taxon",
                excludeTaxa,
                null,
                sourceTaxonNode.getClassification());

        if(targetTaxonNode == null) {
            return new Status(IStatus.CANCEL,
                    "unknown",
                    "");
        }
        if(NavigationUtil.isDirty(targetTaxonNode)){
            return new Status(IStatus.ERROR,
                    "unknown",
                    TaxonNavigatorLabels.UNSAVED_CHANGES_MESSAGE);
        }
        return Status.OK_STATUS;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#prepareOperation(org.eclipse.core.commands.ExecutionEvent)
     */
    @Override
    public AbstractOperation prepareOperation(ExecutionEvent event) {
        return new RemotingMoveFactualDataOperation(event.getTrigger(),
                false,
                sourceTaxonNode.getTaxon().getUuid(),
                targetTaxonNode.getTaxon().getUuid());
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.taxeditor.operation.RemotingCdmHandler#onComplete()
     */
    @Override
    public void onComplete() {
        Display.getDefault().asyncExec(new Runnable(){
            @Override
            public void run() {
                try {
                    //close and re-open to refresh factual data view
                    MultiPageTaxonEditor sourceEditor = (MultiPageTaxonEditor) EditorUtil.findEditorByTaxonNodeUuid(sourceTaxonNode.getUuid());
                    MultiPageTaxonEditor targetEditor = (MultiPageTaxonEditor) EditorUtil.findEditorByTaxonNodeUuid(targetTaxonNode.getUuid());
                    if(targetEditor != null){
                        AbstractUtility.close(sourceEditor);
                        AbstractUtility.close(targetEditor);
                    }
                    EditorUtil.openTaxonNode(sourceTaxonNode.getUuid());
                    EditorUtil.openTaxonNode(targetTaxonNode.getUuid());
                } catch (PartInitException e) {
                    MessagingUtils.error(this.getClass(), e);
                    throw new RuntimeException(e);
                } catch (Exception e) {
                    MessagingUtils.warningDialog("Could not create Taxon", this, e.getMessage());
                }
            }

        });

    }

}
