/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation.navigator.operation;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.application.ICdmApplicationConfiguration;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IClassificationService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.config.TaxonDeletionConfigurator;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.Classification;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.model.DeleteResultMessagingUtils;
import eu.etaxonomy.taxeditor.navigation.internal.TaxeditorNavigationPlugin;
import eu.etaxonomy.taxeditor.operation.AbstractPersistentPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * <p>DeleteTreeNodeOperation class.</p>
 *
 * @author n.hoffmann
 * @created Jan 20, 2010
 * @version 1.0
 */
public class DeleteOperation extends AbstractPersistentPostOperation{

    private Set<ITaxonTreeNode> treeNodes;
    protected final TaxonDeletionConfigurator config;
    private final ICdmEntitySessionEnabled cdmEntitySessionEnabled;

    /**
     * <p>Constructor for DeleteTreeNodeOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
     * @param treeNodes a {@link java.util.Set} object.
     */
    public DeleteOperation(String label, IUndoContext undoContext,
            ITaxonTreeNode taxonNode, TaxonDeletionConfigurator config,
            IPostOperationEnabled postOperationEnabled,
            IConversationEnabled conversationEnabled,
            ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
        super(label, undoContext, postOperationEnabled, conversationEnabled, cdmEntitySessionEnabled);
        this.taxonNode = (ITaxonTreeNode)CdmBase.deproxy(taxonNode, CdmBase.class);
        this.config = config;
        this.cdmEntitySessionEnabled = cdmEntitySessionEnabled;
    }

    /**
     * <p>Constructor for DeleteTreeNodeOperation.</p>
     *
     * @param label a {@link java.lang.String} object.
     * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
     * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
     * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
     * @param treeNodes a {@link java.util.Set} object.
     */
    public DeleteOperation(String label, IUndoContext undoContext,
            Set<ITaxonTreeNode> treeNodes, TaxonDeletionConfigurator config,
            IPostOperationEnabled postOperationEnabled,
            IConversationEnabled conversationEnabled,
            ICdmEntitySessionEnabled cdmEntitySessionEnabled) {
        super(label, undoContext, postOperationEnabled, conversationEnabled, cdmEntitySessionEnabled);
        this.treeNodes = treeNodes;
        this.config = config;
        this.cdmEntitySessionEnabled = cdmEntitySessionEnabled;
    }




	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		bind();
		monitor.worked(20);
		ICdmApplicationConfiguration controller = CdmStore.getCurrentApplicationConfiguration();
		ITaxonNodeService service = controller.getTaxonNodeService();
		if(taxonNode != null && taxonNode instanceof TaxonNode){
			//((TaxonNode) treeNode).delete();

			element = ((TaxonNode)taxonNode).getTaxon();

			DeleteResult result = service.deleteTaxonNode((TaxonNode)taxonNode, config);
			if (result.isError() && !result.getExceptions().isEmpty()){

				DeleteResultMessagingUtils.messageDialogWithDetails(result, "TaxonNode could not be deleted.", TaxeditorNavigationPlugin.PLUGIN_ID);
			} else if (!result.getExceptions().isEmpty()){
				String separator = ", ";
			    String exceptionString = "";
			    for (Exception exception : result.getExceptions()) {
		            exceptionString += exception.getLocalizedMessage()+separator;
		        }
			    DeleteResultMessagingUtils.messageDialogWithDetails(result, "TaxonNode was successfully deleted, but related object(s) could not be deleted", TaxeditorNavigationPlugin.PLUGIN_ID);
			}


		}else if(taxonNode != null && taxonNode instanceof Classification){
			Classification taxonomicTree = (Classification) taxonNode;

			DeleteResult result = CdmStore.getService(IClassificationService.class).delete(taxonomicTree.getUuid());
            if (result.isError() && !result.getExceptions().isEmpty()){
                //TODO:Error message!
                DeleteResultMessagingUtils.messageDialogWithDetails(result, "TaxonNode could not be deleted.", TaxeditorNavigationPlugin.PLUGIN_ID);
            }
        } else {
            Set<UUID> treeNodeUuids = new HashSet<UUID>();
            ITaxonTreeNode entity = null;
            for(ITaxonTreeNode treeNode : treeNodes) {
                if(entity == null) {
                    entity = treeNode;
                }
                treeNodeUuids.add(treeNode.getUuid());
            }

            DeleteResult result = service.deleteTaxonNodes(treeNodeUuids, config);
            if (result.isError() && !result.getExceptions().isEmpty()){
				//TODO:Error message!
                DeleteResultMessagingUtils.messageDialogWithDetails(result, "TaxonNode could not be deleted.", TaxeditorNavigationPlugin.PLUGIN_ID);
			}

        }

        monitor.worked(40);

        return postExecute(null);
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info)
            throws ExecutionException {
        return null;
    }

    /* (non-Javadoc)
     * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info)
            throws ExecutionException {
        return null;
    }
}
